part of '../table_widget.dart';

class _OverlayPortalController extends OverlayPortalController {
  final VoidCallback? onShow;
  final VoidCallback? onHide;

  _OverlayPortalController({
    this.onShow,
    this.onHide,
  });

  @override
  void show() {
    onShow?.call();
    super.show();
  }

  @override
  void hide() {
    super.hide();
    onHide?.call();
  }
}

class AppDateFilter<M extends Object> extends AppFilter<M> {
  final String label;
  final ValueChanged<DateTime?> onChanged;
  final DateTime? initialValue;

  const AppDateFilter({
    required this.label,
    required this.onChanged,
    this.initialValue,
    super.key,
  });

  @override
  State<AppFilter<M>> createState() => _AppDateFilterState<M>();
}

class _AppDateFilterState<M extends Object> extends _AppFilterState<M> {
  @override
  AppDateFilter<M> get widget => super.widget as AppDateFilter<M>;

  late final _dateNotifier = ValueNotifier<DateTime?>(
    widget.initialValue,
  );

  late final _textController = TextEditingController(
    text: _formattedDate,
  );

  late final _overlayController = _OverlayPortalController(
    onHide: _setValue,
  );

  final _link = LayerLink();

  String get _formattedDate {
    if (_dateNotifier.value == null) {
      return '';
    }

    return DateFormat('dd/MM/yyyy').format(_dateNotifier.value!);
  }

  void _dateNotifierListener() {
    _textController.text = _formattedDate;
  }

  void _setValue() {
    widget.onChanged(_dateNotifier.value);
    if (_overlayController.isShowing) {
      _overlayController.hide();
    }
    context.maybeController<M>()?.reload();
  }

  @override
  void initState() {
    super.initState();

    _dateNotifier.addListener(_dateNotifierListener);
  }

  @override
  void dispose() {
    _textController.dispose();
    _dateNotifier.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return CompositedTransformTarget(
      link: _link,
      child: OverlayPortal(
        controller: _overlayController,
        overlayChildBuilder: (context) {
          return CompositedTransformFollower(
            link: _link,
            showWhenUnlinked: false,
            targetAnchor: Alignment.bottomLeft,
            child: Align(
              alignment: Alignment.topLeft,
              child: SizedBox(
                width: 300,
                child: TapRegion(
                  onTapOutside: (_) => _overlayController.hide(),
                  child: Card(
                    child: Padding(
                      padding: const EdgeInsets.all(8),
                      child: CalendarDatePicker(
                        initialDate: _dateNotifier.value ?? DateTime.now(),
                        firstDate: DateTime(2020),
                        lastDate: DateTime(2025),
                        onDateChanged: (date) {
                          _dateNotifier.value = date;
                        },
                      ),
                    ),
                  ),
                ),
              ),
            ),
          );
        },
        child: ListenableBuilder(
          listenable: _textController,
          builder: (context, child) {
            final hasText = _textController.text.isNotEmpty;

            return FilterChip(
              label: Row(
                mainAxisSize: MainAxisSize.min,
                children: [
                  const Icon(Icons.calendar_today, size: 16),
                  const Spacing(),
                  Flexible(
                    child: Text(
                      [
                        widget.label,
                        if (hasText) _textController.text,
                      ].join(': '),
                    ),
                  ),
                ],
              ),
              selected: hasText,
              showCheckmark: false,
              deleteIconColor: context.colorScheme.onPrimary,
              onSelected: (_) {
                _overlayController.show();
              },
              onDeleted: hasText
                  ? () {
                      _dateNotifier.value = null;
                      _setValue();
                    }
                  : null,
            );
          },
        ),
      ),
    );
  }
}
